import geopandas as gpd
import zipfile
import tempfile
from pathlib import Path
import logging

logger = logging.getLogger(__name__)

class GeoProcessor:
    def __init__(self, boundary_file):
        self.boundary_file = Path(boundary_file)
        self.gdf = None
        self._load_boundary()
    
    def _load_boundary(self):
        """Load boundary file (KMZ/GeoJSON/Shapefile)."""
        try:
            if self.boundary_file.suffix.lower() == '.kmz':
                self._load_kmz()
            elif self.boundary_file.suffix.lower() == '.geojson':
                self.gdf = gpd.read_file(self.boundary_file)
            elif self.boundary_file.suffix.lower() in ['.shp', '.zip']:
                self.gdf = gpd.read_file(self.boundary_file)
            else:
                raise ValueError(f"Unsupported file format: {self.boundary_file.suffix}")
            
            # Ensure CRS is WGS84
            if self.gdf.crs != 'EPSG:4326':
                self.gdf = self.gdf.to_crs('EPSG:4326')
                
            logger.info(f"Loaded {len(self.gdf)} features from {self.boundary_file}")
            
        except Exception as e:
            logger.error(f"Failed to load boundary file: {e}")
            raise
    
    def _load_kmz(self):
        """Load KMZ file by extracting and reading KML."""
        with tempfile.TemporaryDirectory() as temp_dir:
            with zipfile.ZipFile(self.boundary_file, 'r') as zip_ref:
                zip_ref.extractall(temp_dir)
            
            # Find KML file
            kml_files = list(Path(temp_dir).glob('*.kml'))
            if not kml_files:
                raise ValueError("No KML file found in KMZ archive")
            
            # Load KML
            gpd.io.file.fiona.drvsupport.supported_drivers['KML'] = 'rw'
            self.gdf = gpd.read_file(kml_files[0], driver='KML')
    
    def get_bounds(self):
        """Get bounding box of all features."""
        if self.gdf is None:
            raise ValueError("No boundary data loaded")
        
        bounds = self.gdf.total_bounds
        return {
            'min_lon': bounds[0],
            'min_lat': bounds[1], 
            'max_lon': bounds[2],
            'max_lat': bounds[3]
        }
    
    def get_geometry(self):
        """Get the combined geometry of all features."""
        if self.gdf is None:
            raise ValueError("No boundary data loaded")
        
        return self.gdf.geometry.unary_union