import requests
import math
import logging
from pathlib import Path
from urllib.parse import urljoin
import time

logger = logging.getLogger(__name__)

class SRTMDownloader:
    def __init__(self, output_dir, temp_dir=None):
        self.output_dir = Path(output_dir)
        self.temp_dir = Path(temp_dir) if temp_dir else self.output_dir / 'temp'
        self.base_url = "https://cloud.sdstate.edu/index.php/s/UjQFkr4y8EGB3JH/download"
        
        # Create directories
        self.output_dir.mkdir(parents=True, exist_ok=True)
        self.temp_dir.mkdir(parents=True, exist_ok=True)
    
    def _get_srtm_tiles(self, bounds):
        """Calculate which SRTM tiles are needed for given bounds."""
        min_lon, min_lat = bounds['min_lon'], bounds['min_lat']
        max_lon, max_lat = bounds['max_lon'], bounds['max_lat']
        
        # SRTM tiles are 1 degree x 1 degree
        tiles = []
        
        for lon in range(math.floor(min_lon), math.ceil(max_lon)):
            for lat in range(math.floor(min_lat), math.ceil(max_lat)):
                # SRTM naming convention: N/S followed by 2-digit lat, E/W followed by 3-digit lon
                lat_str = f"{'N' if lat >= 0 else 'S'}{abs(lat):02d}"
                lon_str = f"{'E' if lon >= 0 else 'W'}{abs(lon):03d}"
                tile_name = f"{lat_str}{lon_str}"
                tiles.append(tile_name)
        
        return tiles
    
    def _download_tile(self, tile_name):
        """Download a single SRTM tile."""
        filename = f"{tile_name}.SRTMGL1.hgt.zip"
        output_path = self.output_dir / filename
        
        if output_path.exists():
            logger.info(f"Tile {tile_name} already exists, skipping")
            return True
        
        # Try multiple SRTM data sources
        urls = [
            f"https://cloud.sdstate.edu/index.php/s/UjQFkr4y8EGB3JH/download?path=%2F&files={filename}",
            f"https://e4ftl01.cr.usgs.gov/MEASURES/SRTMGL1.003/2000.02.11/{filename}"
        ]
        
        for url in urls:
            try:
                logger.info(f"Downloading {tile_name} from {url}")
                
                response = requests.get(url, timeout=300, stream=True)
                response.raise_for_status()
                
                with open(output_path, 'wb') as f:
                    for chunk in response.iter_content(chunk_size=8192):
                        f.write(chunk)
                
                logger.info(f"Successfully downloaded {tile_name}")
                return True
                
            except requests.RequestException as e:
                logger.warning(f"Failed to download {tile_name} from {url}: {e}")
                continue
        
        logger.error(f"Failed to download tile {tile_name} from all sources")
        return False
    
    def download_for_bounds(self, bounds):
        """Download all SRTM tiles needed for given bounds."""
        tiles = self._get_srtm_tiles(bounds)
        
        logger.info(f"Need to download {len(tiles)} SRTM tiles")
        logger.info(f"Tiles: {tiles}")
        
        successful = 0
        failed = 0
        
        for tile in tiles:
            if self._download_tile(tile):
                successful += 1
            else:
                failed += 1
            
            # Be nice to the server
            time.sleep(1)
        
        logger.info(f"Download complete: {successful} successful, {failed} failed")
        
        if failed > 0:
            raise RuntimeError(f"Failed to download {failed} tiles")
        
        return successful